@@ -14,10 +14,10 @@
 #define PG_AUTOCTL_STATE_VERSION 1
 
 /* additional version information for printing version on CLI */
-#define PG_AUTOCTL_VERSION "1.6.4"
+#define PG_AUTOCTL_VERSION "2.0"
 
 /* version of the extension that we requite to talk to on the monitor */
-#define PG_AUTOCTL_EXTENSION_VERSION "1.6"
+#define PG_AUTOCTL_EXTENSION_VERSION "2.0"
 
 /* environment variable to use to make DEBUG facilities available */
 #define PG_AUTOCTL_DEBUG "PG_AUTOCTL_DEBUG"
@@ -34,6 +34,15 @@
 #define PG_AUTOCTL_CANDIDATE_PRIORITY "PG_AUTOCTL_CANDIDATE_PRIORITY"
 #define PG_AUTOCTL_REPLICATION_QUORUM "PG_AUTOCTL_REPLICATION_QUORUM"
 
+/* environment variables for `pg_autoctl do demo run` */
+#define PG_AUTOCTL_DEMO_CLIENTS "PG_AUTOCTL_DEMO_CLIENTS"
+#define PG_AUTOCTL_DEMO_DURATION "PG_AUTOCTL_DEMO_DURATION"
+#define PG_AUTOCTL_DEMO_FAILOVER_FREQ "PG_AUTOCTL_DEMO_FAILOVER_FREQ"
+#define PG_AUTOCTL_DEMO_FAILOVER_FIRST "PG_AUTOCTL_DEMO_FAILOVER_FIRST"
+#define PG_AUTOCTL_SSL_SELF_SIGNED "PG_AUTOCTL_SSL_SELF_SIGNED"
+#define PG_AUTOCTL_AUTH_METHOD "PG_AUTOCTL_AUTH_METHOD"
+#define PG_AUTOCTL_PG_HBA_LAN "PG_AUTOCTL_PG_HBA_LAN"
+
 /* default values for the pg_autoctl settings */
 #define POSTGRES_PORT 5432
 #define POSTGRES_DEFAULT_LISTEN_ADDRESSES "*"
@@ -90,8 +99,12 @@
 #define POSTGRESQL_FAILS_TO_START_TIMEOUT 20
 #define POSTGRESQL_FAILS_TO_START_RETRIES 3
 
+#define CITUS_MASTER_UPDATE_NODE_LOCK_COOLDOWN 10000
+#define CITUS_COORDINATOR_WAIT_MAX_RETRIES 100
+#define CITUS_COORDINATOR_WAIT_TIMEOUT 300
 #define DEFAULT_CITUS_ROLE "primary"
 #define DEFAULT_CITUS_CLUSTER_NAME "default"
+#define DEFAULT_CITUS_DBNAME "citus"
 
 #define FAILOVER_FORMATION_NUMBER_SYNC_STANDBYS 1
 #define FAILOVER_NODE_CANDIDATE_PRIORITY 50
