@@ -16,6 +16,12 @@
 #include <time.h>
 #include <unistd.h>
 
+#include "postgres.h"
+
+#if PG_MAJORVERSION_NUM >= 15
+#include "common/pg_prng.h"
+#endif
+
 #include "cli_do_demoapp.h"
 #include "defaults.h"
 #include "demoapp.h"
@@ -144,10 +150,15 @@ demoapp_prepare_schema(const char *pguri)
 	const char *ddls[] = {
 		"drop schema if exists demo cascade",
 		"create schema demo",
+
+		"create table demo.client(client integer primary key, pid integer, "
+		"retry_sleep_ms integer, retry_cap_ms integer, failover_count integer, "
+		"unique(pid))",
+
 		"create table demo.tracking(ts timestamptz default now(), "
-		"client integer, loop integer, retries integer, us bigint, recovery bool)",
-		"create table demo.client(client integer, pid integer, "
-		"retry_sleep_ms integer, retry_cap_ms integer, failover_count integer)",
+		"client integer, loop integer, retries integer, us bigint, recovery bool,"
+		"primary key(client, ts),"
+		"foreign key (client) references demo.client(client))",
 		NULL
 	};
 
@@ -377,9 +388,12 @@ demoapp_process_perform_switchover(DemoAppOptions *demoAppOptions)
 
 	if (demoAppOptions->duration <= (demoAppOptions->firstFailover + 10))
 	{
+		log_warn("Using --duration %ds and --first-failover %ds",
+				 demoAppOptions->duration,
+				 demoAppOptions->firstFailover);
 		log_error("Use a --duration of at least %ds for a failover to happen",
 				  demoAppOptions->firstFailover + 10);
-		exit(EXIT_CODE_INTERNAL_ERROR);
+		exit(EXIT_CODE_QUIT);
 	}
 
 	log_info("Failover client is started, will failover in %ds "
@@ -530,8 +544,13 @@ demoapp_update_client_failovers(const char *pguri, int clientId, int failovers)
 /*
  * http://c-faq.com/lib/randrange.html
  */
+#if PG_MAJORVERSION_NUM < 15
 #define random_between(M, N) \
 	((M) + pg_lrand48() / (RAND_MAX / ((N) -(M) +1) + 1))
+#else
+#define random_between(M, N) \
+	((M) + pg_prng_uint32(&prng_state) / (RAND_MAX / ((N) -(M) +1) + 1))
+#endif
 
 /*
  * demo_start_client starts a sub-process that implements our demo application:
@@ -558,7 +577,14 @@ demoapp_start_client(const char *pguri, int clientId,
 	int retrySleepTime = 500;   /* first retry happens after 500 ms */
 
 	/* initialize a seed for our random number generator */
+
+#if PG_MAJORVERSION_NUM < 15
 	pg_srand48(((unsigned int) (getpid() ^ time(NULL))));
+#else
+	pg_prng_state prng_state;
+
+	pg_prng_seed(&prng_state, (uint64) (getpid() ^ time(NULL)));
+#endif
 
 	/* pick a random retry policy for this client */
 	retryCap = random_between(50, 500);
